require 'soar/registry/directory/error'
require 'soar/registry/directory/provider/stub'
require 'soar/registry/directory/provider/dynamo_db'
require 'soar/registry/directory/provider/ldap'
require 'soar/registry/directory/provider/mysql'

module Soar
  module Registry
    module Directory
      class Model 

        attr_reader :provider

        ##
        # @param [Soar::Registry::Directory::Provider::Stub, Soar::Registry::Directory::Provider::DynamoDb, Soar::Registry::Directory::Provider::Ldap, Soar::Registry::Directory::Provider::Mysql] provider
        ##
        def initialize(provider)
          @provider = provider
        end

        ##
        # @param [Hash{String => }] entity
        # @return [Boolean]
        # @raise [Soar::Registry::Directory::Error::NetworkingError, Soar::Registry::Directory::Error::DuplicateEntryError]
        ##
        def put(entity)
          @provider.put(entity)
        end

        ##
        # @param [String] primary_key
        # @return [Hash{String => }] entry
        # @raise [Soar::Registry:::Directory::Error::NoEntriesFound] if primary key not found
        # @raise [Soar::Registry::Directory::Error::NetworkingError]
        ##
        def fetch(primary_key)
          @provider.fetch(primary_key)
        end

        ##
        # @param [String] key 
        # @param [String] value
        # @return [Array<Hash{String => }>] list of entries
        # @raise [ArgumentError, Soar::Registry::Directory::Error::NetworkingError]
        ##
        def search(key, value)
          @provider.search(key, value)
        end

        ##
        # @return [Array<String>] a list of provider specific indexes
        # @raise [Soar::Registry::Directory::Error::NetworkingError]
        ##
        def index
          @provider.index
        end

      end
    end
  end
end
