require 'soar/registry/directory'
require 'aws-sdk'
require 'soar/registry/directory/error'

module Soar
  module Registry
    module Directory
      module Test
        module Provider
          class DynamoDb

            def initialize
              @retries = 5
              @configuration = YAML.load_file("config/#{ENV['CONFIG_FILE']}")
              @table = @configuration['provider']['config']['table']
              @index = @table['index']
              @entries = JSON.parse(File.read("lib/soar/registry/directory/test/fixtures/entries.json"))
              @connection_retries = 6
            end

            def given_configured_directory
              begin 
                provider = Soar::Registry::Directory::Provider::DynamoDb.new(@configuration['provider']['config'].map { |k, v| [k.to_sym, v] }.to_h)
                @directory = Soar::Registry::Directory.new(provider)
                @directory.provider.recreate_table({
                  name: @table['name'], 
                  structure: JSON.parse(File.read("lib/soar/registry/directory/test/fixtures/table_structure.json"))
                })
              rescue Soar::Registry::Directory::Error::NetworkingError => e
                raise if @connection_retries == 0
                sleep 10
                @connection_retries -= 1
                given_configured_directory
              end
            end

            def given_existing_data
              @entries.each{ |entry| 
                @directory.put(entry)
              }
            end

            def sabotage_network
              @configuration['provider']['config']['configuration']['endpoint'] = 'http://does-not-exist'
              @directory.provider.client = Aws::DynamoDB::Client.new(@configuration['provider']['config']['configuration'].map { |k, v| [k.to_sym, v] }.to_h)
            end

            def put_duplicate_entry
              put_entry
            end

            def put_entry
              begin
                @directory.put(@entries[0])
              rescue Soar::Registry::Directory::Error::NetworkingError => e
                @error = e
              rescue Soar::Registry::Directory::Error::DuplicateEntryError => e
                @error = e
              end
            end

            def search_for_entry
              begin
                @result = @directory.search("global_secondary_index1", @entries[0]['global_secondary_index1'])
              rescue Soar::Registry::Directory::Error::NetworkingError => e
                @error = e
              end
            end

            def request_index
              begin
                @result = @directory.index
              rescue Soar::Registry::Directory::Error::NetworkingError => e
                @error = e
              end
            end

            def fetch_entry
              begin
                @result = @directory.fetch({
                  @index.keys[0] => @entries[0]['partition_key'],
                  @index.keys[1] => @entries[0]['sort_key']
                })
              rescue Soar::Registry::Directory::Error::NoEntriesFoundError, Soar::Registry::Directory::Error::NetworkingError => e
                @error = e
              end
            end

            def duplicate_entry?
              @error.is_a?(Soar::Registry::Directory::Error::DuplicateEntryError)
            end

            def persisted?
              @entries[0] == @directory.fetch({
                  @index.keys[0] => @entries[0]['partition_key'],
                  @index.keys[1] => @entries[0]['sort_key']
                })
            end

            def returned?
              @result == [@entries[0]]
            end

            def index?
              @index == {
                "partition_key" => @index['partition_key'],
                "sort_key" => @index['sort_key'],
                "global_secondary_index" => @index['global_secondary_index']
              }
            end

            def single_entry?
              @result == @entries[0]
            end

            def no_entries_found?
              @error.is_a?(Soar::Registry::Directory::Error::NoEntriesFoundError)
            end

            def networking_error?
              @error.is_a?(Soar::Registry::Directory::Error::NetworkingError)
            end

            def no_matching_entries?
              @result == []
            end

          end
        end
      end
    end

  end
end
