require 'soar/registry/directory'
require 'mince'
require 'hashy_db'

module Soar
  module Registry
    module Directory
      module Test
        module Provider
          class Stub

            @@entry = {
              "uuid" => SecureRandom.uuid,
              "identifier" => SecureRandom.hex
            }

            @@index = ["uuid", "identifier"]

            def given_configured_directory
              provider = Soar::Registry::Directory::Provider::Stub.new({
                table: "identities",
                index: @@index
              })
              @directory = Soar::Registry::Directory.new(provider)
            end

            def given_existing_data
              @directory.put(@@entry)
            end

            def sabotage_network
              @directory.provider.instance_variable_set(:@interface, Object.new)
            end

            def put_duplicate_entry
              put_entry
            end

            def put_entry
              begin
                @directory.put(@@entry)
              rescue Soar::Registry::Directory::Error::NetworkingError => e
                @error = e
              rescue Soar::Registry::Directory::Error::DuplicateEntryError => e
                @error = e
              end
            end

            def search_for_entry
              begin
                @entry = @directory.search("identifier", @@entry['identifier'])
              rescue Soar::Registry::Directory::Error::NetworkingError => e
                @error = e
              end
            end

            def request_index
              begin
                @index = @directory.index
              rescue Soar::Registry::Directory::Error::NetworkingError => e
                @error = e
              end
            end

            def fetch_entry
              begin
                @entry = @directory.fetch(@@entry['uuid'])
              rescue Soar::Registry::Directory::Error::NoEntriesFoundError, Soar::Registry::Directory::Error::NetworkingError => e
                @error = e
              end
            end

            def duplicate_entry?
              @error.is_a?(Soar::Registry::Directory::Error::DuplicateEntryError)
            end

            def persisted?
              @@entry == @directory.fetch(@@entry["uuid"])
            end

            def returned?
              @entry == [@@entry]
            end

            def index?
              @index.is_a?(Array)
            end

            def single_entry?
              @entry == @@entry
            end

            def no_entries_found?
              @error.is_a?(Soar::Registry::Directory::Error::NoEntriesFoundError)
            end

            def no_matching_entries?
              @entry == []
            end

            def networking_error?
              @error.is_a?(Soar::Registry::Directory::Error::NetworkingError)
            end

          end
        end
      end
    end

  end
end
